package com.engine;

import java.text.DecimalFormat;
import java.util.List;

import mesure.Coordonnee;
import mesure.Node;
import mesure.NodeP;
import mesure.Relation;
import mesure.Type_Tag;
import mesure.Ways;
import sensor.Compass;
import sensor.GPSListener;
import userinterface.MainActivity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.location.LocationProvider;
import android.os.Bundle;
import android.text.InputType;
import android.util.Log;
import android.widget.EditText;
import android.widget.ListView;

import com.mapquest.android.maps.GeoPoint;
import com.mapquest.android.maps.OverlayItem;

import communication.I_Communication;
import communication.WF_Communication;

public class Engine {
	
	private MainActivity mainActivity;
	private I_Communication sensor_ext;
	private Bundle savedInstanceState;
	private PaserOSM parser;
	private LocationListener locationLister;
	private LocationManager locationManager;
	private Ways currentway ;
	private Relation r;
	private Compass compass;
	private Coordonnee initialPoint;

	private Location lInit;
	private Ellipsoid ellipsoid;
	private int nbtourprec = 0;
	private int nbtourprec_init = 0 ;
	private String name_st;
	private NodeP initalNode;
	private double distance_pause = 0 ;

	private String poivalue ="defaultPOI" ;
	private String[] stringArray;
	private String value ;
	private DecimalFormat df;
	private boolean measuring = false;
	
	public Engine(MainActivity main,Bundle s){
		
		df = new DecimalFormat("########.00");
		mainActivity = main;
		savedInstanceState = s;
		parser = new PaserOSM(this);
		ellipsoid = Ellipsoid.WGS84; 
		setR(new Relation(mainActivity.getId_relation()));
	}
	
	/**
	 * Init the engine
	 * Sensor used : compass
	 * intialize the initial point
	 * @param c
	 * @param name
	 */
	public void init(Coordonnee c, String name){
		initialPoint = c ;
		name_st = name;
		
		if(getBouss() == null){
	    	setBouss(new Compass(mainActivity,this,mainActivity.getdirection_dep()));	
	    }	    
	    getBouss().start();   
	    mainActivity.swapFrag("Mesure");
	}
	
	/**
	 * Init the engine
	 * Sensor used : compass
	 * intialize the initial point
	 * @param nodeP
	 */
	public void init(NodeP nodeP) {
		
		initalNode = nodeP;
		
		if(getBouss() == null){
	    	setBouss(new Compass(mainActivity,this,mainActivity.getdirection_dep()));	
	    }	    
	    getBouss().start();
	    
		mainActivity.swapFrag("Mesure");
	}
	/**
	 * Start the measure
	 * 
	 * Initialize the initial point if is not defined
	 * Initialize the sensor out which implement I_Communication
	 * Initialize the tour count 
	 * Create the first node in the way
	 */
	public void start(){
		measuring = true;
		if(sensor_ext != null){
			updateDisplayDistance();
			distance_pause = updtateDistance();
			mainActivity.TextDistance("0,00 m ");	
		}

		if(lInit != null)
		{
			initialPoint = new Coordonnee(lInit.getLatitude(), lInit.getLongitude());
		}
		//mainActivity.alert("Debug", "Go ma poule ! ");
		if(sensor_ext == null){
			 sensor_ext = new WF_Communication(mainActivity, savedInstanceState);
			 sensor_ext.init();
			 
		}else{
			sensor_ext.start();
		}
		nbtourprec_init = nbtourprec;
	    
	    currentway = new Ways(mainActivity.getId_way()); 

	    if(initalNode != null){
	    	currentway.addNode(initalNode);
	    }
	    
	    if(name_st != null){
	    	NodeP np = new NodeP(mainActivity.getId_node(),name_st, initialPoint, Type_Tag.name);
	    	currentway.addNode(np);
	    	if(!np.getName().equals(""))
	    	{
	    		mainActivity.saveNode(np);
	    	}
	    }
	    //Compass valid value
	   	getBouss().set_start(true);
	}
	
	/**
	 * Stop the measure , but not write in the file save. 
	 * Turn off the compass and invalidate value
	 * Get the final point name
	 */
	public void stop(){
		getBouss().stop();
		getBouss().set_start(false);
		final Coordonnee c = calcul_new_coordonnees(currentway.getNodes().get(currentway.getNodes().size()-1).getCoordonnee(), getBouss().getBearing(), getDistance());
		AlertDialog.Builder builder = new AlertDialog.Builder(mainActivity);
		builder.setTitle("Nom du point final");

		// Set up the input
		final EditText input = new EditText(mainActivity);
		// Specify the type of input expected; this, for example, sets the input as a password, and will mask the text
		input.setInputType(InputType.TYPE_CLASS_TEXT);
		builder.setView(input);

		// Set up the buttons
		builder.setPositiveButton("OK", new DialogInterface.OnClickListener() { 
			@Override
			public void onClick(DialogInterface dialog, int which) {
				NodeP np = new NodeP(mainActivity.getId_node(), input.getText().toString() ,c , Type_Tag.name);
				currentway.addNode(np);
				if(!np.getName().equals(""))
				{
					mainActivity.saveNode(np);
				}
				
				getR().addWay(currentway);
				updtateDistance();

				//paser.export_xml(r);
				mainActivity.swapFrag("Init");
				return;
			}
		});
		builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
				return;
			}
		});
		builder.create();
		builder.show();
		//Measure end
		measuring = false ;
	}
	
	/**
	 * Save a POI. The user can be choose the type and name of POI
	 * Get the POI type and name
	 * Adding the POI on the map
	 * 
	 */
	public void savePOI()
	{
		final Coordonnee c = calcul_new_coordonnees(currentway.getNodes().get(currentway.getNodes().size()-1).getCoordonnee(), getBouss().getBearing(), getDistance());
		AlertDialog.Builder builder = new AlertDialog.Builder(mainActivity);
		builder.setTitle("Select type of POI");
		ListView modeList = new ListView(mainActivity);
		stringArray = new String[] { Type_Tag.name.toString(),Type_Tag.toilets.toString(),Type_Tag.stair.toString(),Type_Tag.door.toString()  };
		builder.setItems(stringArray, new DialogInterface.OnClickListener() {
             public void onClick(DialogInterface dialog, int which) {		            	 
            	
            	value =  stringArray[which];
         		AlertDialog.Builder builder = new AlertDialog.Builder(mainActivity);
            	builder.setTitle("Nom du point");

            	// Set up the input
            	final EditText input = new EditText(mainActivity);
            	// Specify the type of input expected; this, for example, sets the input as a password, and will mask the text
            	input.setInputType(InputType.TYPE_CLASS_TEXT);
            	builder.setView(input);
            	
            	// Set up the buttons
            	builder.setPositiveButton("OK", new DialogInterface.OnClickListener() { 
            	    @Override
            	    public void onClick(DialogInterface dialog, int which) {
                    	
                    	poivalue= input.getText().toString();
                    	GeoPoint geopoint = new GeoPoint(c.get_lat(),c.get_long());
                		OverlayItem o = new OverlayItem(geopoint, poivalue, "");
                		mainActivity.getOverlay().addItem(o);
                 		NodeP np = new NodeP(mainActivity.getId_node(),poivalue , c, Type_Tag.valueOf(value));
                 		currentway.addNode(np);
            	    	if(!np.getName().equals(""))
            	    	{
            	    		mainActivity.saveNode(np);
            	    	}
                 		updtateDistance();
                 		getR().addWay(currentway);
                 		currentway = new Ways(mainActivity.getId_way());
                 		currentway.addNode(np);
            	       	return;
            	    }
            	});
            	builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            	    @Override
            	    public void onClick(DialogInterface dialog, int which) {
            	        dialog.cancel();
            	        return;
            	    }
            	});
            	builder.create();
            	builder.show();

	           }
	    });
		builder.setView(modeList);
		final Dialog dialog = builder.create();
 		dialog.show();
	}
	
	/**
	 * Turn off the sensors and save the savedNode in backup file
	 */
	public void finish(){
		parser.export_xml_config(mainActivity.getSavedNode());
		if(sensor_ext!=null)
		sensor_ext.finish();
		if(getBouss()!=null)
		getBouss().stop();
	}

	/**
	 * 
	 * @return the list of saved node
	 */
	public List<NodeP> getSavedNode() {
		return mainActivity.getSavedNode();
	}

	/**
	 * 
	 * @return currentway
	 */
	public Ways getCurrentWay(){
		return currentway;
	}
	
	/**
	 * 
	 * @param degree
	 */
	public void new_node(float degree){
		currentway.addNode(new Node(mainActivity.getId_node(), calcul_new_coordonnees(currentway.getNodes().get(currentway.getNodes().size()-1).getCoordonnee(), degree, updtateDistance())));
		
	}
	
	/**
	 * Update the compass Displaying
	 * @param s
	 */
	public void updateDisplayDirection(String s){
		mainActivity.Textboussole(s);
	}
	
	/**
	 * Display the distance in the user interface
	 */
	public void updateDisplayDistance(){
		try{
			double d = (sensor_ext.getTour()- nbtourprec_init) * mainActivity.getCirconference();
			String distance = df.format(d);
			d = Double.parseDouble(distance.replace(',', '.'));
			mainActivity.TextDistance("Distance : "+ distance);
		}catch(Exception e){
			
		}

	}
	
	/**
	 * 
	 * @return the current distance and  (update to create a new value)
	 */
	private double updtateDistance() {
		//Log.d("Distance",""+(c.getTour()-nbtourprec));
		double temp = (sensor_ext.getTour()- nbtourprec) * mainActivity.getCirconference();
		nbtourprec = sensor_ext.getTour() ;
		Log.d("Distance prec in distance",""+temp+"::"+nbtourprec);
		return temp ;
	}
	
	/**
	 * 
	 * @return the current distance (not update)
	 */
	private double getDistance(){
		return (sensor_ext.getTour()- nbtourprec -distance_pause) * mainActivity.getCirconference();
	}
	
	/**
	 * GPS Initialisation  start point
	 * @param s
	 */
	public void initGPS(String s) {
		locationManager = (LocationManager) mainActivity.getSystemService(MainActivity.LOCATION_SERVICE);
		if ( !locationManager.isProviderEnabled( LocationManager.GPS_PROVIDER ) ) {
	        Log.i("GPS","Desactivate"); 
			buildAlertMessageNoGps();
	    }
		else
		{
			locationLister = new GPSListener(mainActivity, this);
			
			Criteria c = new Criteria();
			  c.setAccuracy(Criteria.ACCURACY_FINE);
			  c.setAltitudeRequired(false);
			  c.setBearingRequired(false);
			  c.setSpeedRequired(false);
			  c.setCostAllowed(true);
			  c.setPowerRequirement(Criteria.POWER_HIGH);
			LocationProvider high= locationManager.getProvider(locationManager.getBestProvider(c,true));

			locationManager.requestLocationUpdates(high.getName(),10000,0,locationLister);
			do{
				if(lInit !=null)
				Log.d("Accuraty",""+lInit.getAccuracy());
			lInit = locationManager.getLastKnownLocation(high.getName());
			
			}
			while(lInit == null|| lInit.getAccuracy() <10 ) ;
			Log.d("Result",""+lInit.getLatitude() +"::"+lInit.getLongitude());
			Log.d("Accuraty",""+lInit.getAccuracy());
			locationManager.removeUpdates(locationLister);
			Log.d("Result",s);
			
			GeoPoint geopoint = new GeoPoint(lInit.getLatitude(),lInit.getLongitude());
			OverlayItem o = new OverlayItem(geopoint, poivalue, "");
			mainActivity.getOverlay().addItem(o);
			this.init(new Coordonnee(lInit.getLatitude(), lInit.getLongitude()),s);
		}
		
	}
	
	/**
	 * Error message if GPS is not enable
	 */
	  private void buildAlertMessageNoGps() {
		    final AlertDialog.Builder builder = new AlertDialog.Builder(mainActivity);
		    builder.setMessage("Your GPS seems to be disabled, do you want to enable it?")
		           .setCancelable(false)
		           .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
		               public void onClick(final DialogInterface dialog, final int id) {
		                   mainActivity.startActivity(new Intent(android.provider.Settings.ACTION_LOCATION_SOURCE_SETTINGS));
		               }
		           })
		           .setNegativeButton("No", new DialogInterface.OnClickListener() {
		               public void onClick(final DialogInterface dialog, final int id) {
		                    dialog.cancel();
		               }
		           });
		    final AlertDialog alert = builder.create();
		    alert.show();
	  }
	  
	  /**
	   * Application of Vincentys-formula
	   * angle en degree , distance en mtre
	   * @param start
	   * @param startBearing
	   * @param distance
	   * @return
	   */
    
	public Coordonnee calcul_new_coordonnees(Coordonnee start, double startBearing, double distance){
				
		  //mainActivity.alert("Distance in nex coordonnate", ""+distance);
	      double a = ellipsoid.getSemiMajorAxis();
	      double b = ellipsoid.getSemiMinorAxis();
	      double aSquared = a * a;
	      double bSquared = b * b;
	      double f = ellipsoid.getFlattening();
	      double phi1 = Angle.toRadians(start.get_lat());
	      double alpha1 = Angle.toRadians(startBearing);
	      double cosAlpha1 = Math.cos(alpha1);
	      double sinAlpha1 = Math.sin(alpha1);
	      double s = distance+1;
	      double tanU1 = (1.0 - f) * Math.tan(phi1);
	      double cosU1 = 1.0 / Math.sqrt(1.0 + tanU1 * tanU1);
	      double sinU1 = tanU1 * cosU1;
	
	      // eq. 1
	      double sigma1 = Math.atan2(tanU1, cosAlpha1);
	
	      // eq. 2
	      double sinAlpha = cosU1 * sinAlpha1;
	
	      double sin2Alpha = sinAlpha * sinAlpha;
	      double cos2Alpha = 1 - sin2Alpha;
	      double uSquared = cos2Alpha * (aSquared - bSquared) / bSquared;
	
	      // eq. 3
	      double A = 1 + (uSquared / 16384) * (4096 + uSquared * (-768 + uSquared * (320 - 175 * uSquared)));
	
	      // eq. 4
	      double B = (uSquared / 1024) * (256 + uSquared * (-128 + uSquared * (74 - 47 * uSquared)));
	
	      // iterate until there is a negligible change in sigma
	      double deltaSigma;
	      double sOverbA = s / (b * A);
	      double sigma = sOverbA;
	      double sinSigma;
	      double prevSigma = sOverbA;
	      double sigmaM2;
	      double cosSigmaM2;
	      double cos2SigmaM2;
	
	      for (;;)
	      {
	         // eq. 5
	         sigmaM2 = 2.0 * sigma1 + sigma;
	         cosSigmaM2 = Math.cos(sigmaM2);
	         cos2SigmaM2 = cosSigmaM2 * cosSigmaM2;
	         sinSigma = Math.sin(sigma);
	         double cosSignma = Math.cos(sigma);
	
	         // eq. 6
	         deltaSigma = B
	               * sinSigma
	               * (cosSigmaM2 + (B / 4.0)
	                     * (cosSignma * (-1 + 2 * cos2SigmaM2) - (B / 6.0) * cosSigmaM2 * (-3 + 4 * sinSigma * sinSigma) * (-3 + 4 * cos2SigmaM2)));
	
	         // eq. 7
	         sigma = sOverbA + deltaSigma;
	
	         // break after converging to tolerance
	         if (Math.abs(sigma - prevSigma) < 0.0000000000001) break;
	
	         prevSigma = sigma;
	      }
	
	      sigmaM2 = 2.0 * sigma1 + sigma;
	      cosSigmaM2 = Math.cos(sigmaM2);
	      cos2SigmaM2 = cosSigmaM2 * cosSigmaM2;
	
	      double cosSigma = Math.cos(sigma);
	      sinSigma = Math.sin(sigma);
	
	      // eq. 8
	      double phi2 = Math.atan2(sinU1 * cosSigma + cosU1 * sinSigma * cosAlpha1, (1.0 - f)
	            * Math.sqrt(sin2Alpha + Math.pow(sinU1 * sinSigma - cosU1 * cosSigma * cosAlpha1, 2.0)));
	
	      double lambda = Math.atan2(sinSigma * sinAlpha1, (cosU1 * cosSigma - sinU1 * sinSigma * cosAlpha1));
	
	      // eq. 9
	      double C = (f / 16) * cos2Alpha * (4 + f * (4 - 3 * cos2Alpha));
	
	      // eq. 10
	      double L = lambda - (1 - C) * f * sinAlpha * (sigma + C * sinSigma * (cosSigmaM2 + C * cosSigma * (-1 + 2 * cos2SigmaM2)));
	
	      // build result
	      double latitude = Angle.toDegrees(phi2);
	      double longitude = start.get_long() + Angle.toDegrees(L);
	      
	      return new Coordonnee(latitude, longitude);

	}

	/**
	 * True if in measure
	 * @return boolean measuring
	 */
	public boolean isMeasuring() {
		return measuring;
	}
	/**
	 * 
	 * @return the mainActivity instance
	 */
	public MainActivity getMainActivity(){
		return mainActivity;
	}
	
	/**
	 * @return the parser
	 */
	public PaserOSM getParser() {
		return parser;
	}

	/**
	 * @param parser the parser to set
	 */
	public void setParser(PaserOSM parser) {
		this.parser = parser;
	}

	/**
	 * @return the initalNode
	 */
	public NodeP getInitalNode() {
		return initalNode;
	}

	/**
	 * @param initalNode the initalNode to set
	 */
	public void setInitalNode(NodeP initalNode) {
		this.initalNode = initalNode;
	}
	
	/**
	 * @return the initialPoint
	 */
	public Coordonnee getInitialPoint() {
		return initialPoint;
	}

	/**
	 * @param initialPoint the initialPoint to set
	 */
	public void setInitialPoint(Coordonnee initialPoint) {
		this.initialPoint = initialPoint;
	}

	/**
	 * @return the bouss
	 */
	public Compass getBouss() {
		return compass;
	}

	/**
	 * @param bouss the bouss to set
	 */
	public void setBouss(Compass bouss) {
		this.compass = bouss;
	}

	/**
	 * @return the r
	 */
	public Relation getR() {
		return r;
	}

	/**
	 * @param r the r to set
	 */
	public void setR(Relation r) {
		this.r = r;
	}

}


