package com.engine;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import mesure.Coordonnee;
import mesure.Node;
import mesure.NodeP;
import mesure.Relation;
import mesure.Tag_OSM;
import mesure.Type_Tag;

import android.text.format.DateFormat;
import android.util.Log;

public class PaserOSM {

	private Engine e ;
	/**
	 * 
	 * @param engine
	 */
	public PaserOSM(Engine engine){
		e = engine;
	}
	
	/**
	 * 	Parses the specified string as Type_tag value. 
	 * @param type  the string representation of an Type_tag value.
	 * @return an Type_Tag instance containing the integer value represented by string.
	 */
	public Type_Tag transform (String type){
		
		if(type.equals("name"))
			return Type_Tag.name;
		if(type.equals("toilets"))
			return Type_Tag.toilets;
		if(type.equals("stairs"))
			return Type_Tag.stair;
		if(type.equals("user"))
			return Type_Tag.user;
		else
			return Type_Tag.user;
	}
	
	/**
	 * Read the XML configuration file contains node
	 * @return list with node
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public List<NodeP> import_xml_config() throws XmlPullParserException, IOException{
		
		XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
		factory.setNamespaceAware(true);
		Document document = null;
		Element racine ;
		
		List<NodeP> list_node = new ArrayList<NodeP>();
		
		SAXBuilder sxb = new SAXBuilder();
		try
		{
		   //On cre un nouveau document JDOM avec en argument le fichier XML
		   document = sxb.build(new File("./mnt/sdcard/Trodometre/BackupNode.xml"));
		   Log.d("OK", "Fichier ouvert");
		}
		catch(Exception e){
			Log.d("Problme", "Fichier non ouvert");
			return list_node;}
		
		racine = document.getRootElement();
		
		List liste_noeud = racine.getChildren("node");
		Log.d("liste node", liste_noeud.get(0).toString());
		//On cre un Iterator sur notre liste
		Iterator i = liste_noeud.iterator();
		while(i.hasNext())
		{
			Element courant = (Element)i.next();
			Log.d("node", courant.getAttributeValue("id"));
			int tmp_id = Integer.valueOf(courant.getAttributeValue("id"));
			Coordonnee tmp_c = new Coordonnee(Double.valueOf(courant.getAttributeValue("lat")), Double.valueOf(courant.getAttributeValue("lon")));
			
			List tag = courant.getChildren("tag");
			Iterator j = tag.iterator();
			Element courant2 = (Element)j.next();
			String tmp_name = courant2.getAttributeValue("v");
			String tmp_type = courant2.getAttributeValue("k");
			NodeP n = new NodeP(tmp_id, tmp_name, tmp_c, transform(tmp_type));
			while(j.hasNext()){
				courant2 = (Element)j.next();
				n.addTags(new Tag_OSM(courant2.getAttributeValue("v"), transform(courant2.getAttributeValue("k"))));
			}			
			list_node.add(n);
		}
		
 		return list_node;	
	}
	
	/**
	 * Write the XML config based on a node list
	 * @param p node list
	 */
	public void export_xml_config(List<NodeP> p){
	 
		Element osm = new Element("osm");

	    //On cre un nouvel Attribut et on l'ajoute  osm grce  la mthode setAttribute
	    Attribute version = new Attribute("version","0.6");
	    osm.setAttribute(version);
	    Attribute upload = new Attribute("upload","true");
	    osm.setAttribute(upload);
	    Attribute generator = new Attribute("generator","Trodometre");
	    osm.setAttribute(generator);
	     
		//On cre un nouveau Document JDOM bas sur la racine que l'on vient de crer
		Document document = new Document(osm);
		 
		//Ajout de la partie noeuds dans le xml
		for(int i=0; i<p.size();i++){
			Element node = new Element("node");
			 
			//Ajout des attributs d'un noeuds
			Attribute id = new Attribute("id",Integer.toString(p.get(i).getId()));
			node.setAttribute(id);
			Attribute action = new Attribute("action","modify");
			//r.getNodes().get(i).getAction().toString();
			node.setAttribute(action);
			Attribute visible = new Attribute("visible","true");			 
			node.setAttribute(visible);
			Attribute lat = new Attribute("lat",Double.toString(p.get(i).getCoordonnee().get_lat()));			 
			node.setAttribute(lat);
			Attribute lon = new Attribute("lon",Double.toString(p.get(i).getCoordonnee().get_long()));			 
			node.setAttribute(lon);
			
			//Ajout des tags pour les noeuds particuliers
			if(p.get(i) instanceof NodeP){
				NodeP np = (NodeP) p.get(i);
				for(int j=0; j<np.getTags().size();j++){
					 Element tag = new Element("tag");
					 Attribute k = new Attribute("k",np.getTags().get(j).getType().toString());
					 tag.setAttribute(k);
					 Attribute v = new Attribute("v",np.getTags().get(j).getValue());
					 tag.setAttribute(v);
					 node.addContent(tag);
				 }
			}
			osm.addContent(node);			 
		    
		    enregistre("./mnt/sdcard/Trodometre/BackupNode.xml", document);
		 }
	}
	
	/**
	 * Write the XML based on a relation
	 * @param r a relation with ways and node
	 */
	public void export_xml(Relation r){
		 
		 Element osm = new Element("osm");

	     //On cre un nouvel Attribut et on l'ajoute  osm grce  la mthode setAttribute
	     //<osm version='0.6' upload='true' generator='JOSM'>
	     Attribute version = new Attribute("version","0.6");
	     osm.setAttribute(version);
	     Attribute upload = new Attribute("upload","true");
	     osm.setAttribute(upload);
	     Attribute generator = new Attribute("generator","Trodometre");
	     osm.setAttribute(generator);
	      
		 //On cre un nouveau Document JDOM bas sur la racine que l'on vient de crer
		 Document document = new Document(osm);
	      
		 //Ajout de la partie noeuds dans le xml
		 for(int i=0; i<r.getNodes().size();i++){
			 Element node = new Element("node");
			 
			 //list_node = r.getNodes()
			 //boucle sur liste de r
			 //On concatne les listes avec un parcours 
			 //Boucle sur la liste de la relation courante rc
			 //if(list_node. contains r.getNodes.get(i))
			 //		on ajoute dans la liste
			 //			 
			 
			 //Ajout des attributs d'un noeuds
			 Attribute id = new Attribute("id",Integer.toString(r.getNodes().get(i).getId()));
			 node.setAttribute(id);
			 Attribute action = new Attribute("action","modify");
			 		//r.getNodes().get(i).getAction().toString();
			 node.setAttribute(action);
			 Attribute visible = new Attribute("visible","true");			 
			 node.setAttribute(visible);
			 Attribute lat = new Attribute("lat",Double.toString(r.getNodes().get(i).getCoordonnee().get_lat()));			 
			 node.setAttribute(lat);
			 Attribute lon = new Attribute("lon",Double.toString(r.getNodes().get(i).getCoordonnee().get_long()));			 
			 node.setAttribute(lon);
			 
			 //Ajout des tags pour les noeuds particuliers
			 if(r.getNodes().get(i) instanceof NodeP){
				 NodeP np = (NodeP) r.getNodes().get(i);
				 for(int j=0; j<np.getTags().size();j++){
					 Element tag = new Element("tag");
					 Attribute k = new Attribute("k",np.getTags().get(j).getType().toString());
					 tag.setAttribute(k);
					 Attribute v = new Attribute("v",np.getTags().get(j).getValue());
					 tag.setAttribute(v);
					 node.addContent(tag);
				 }
			 }
			 osm.addContent(node);
		 }
		 
		 //Ajout de la partie chemin dans le xml
		 for(int i=0; i<r.getWays().size();i++){
			 Element way = new Element("way");
			 
			 //Ajout des attributs d'un chemin
			 Attribute id = new Attribute("id",Integer.toString(r.getWays().get(i).getId()));
			 way.setAttribute(id);
			 Attribute action = new Attribute("action","modify");
			 		//r.getNodes().get(i).getAction().toString();
			 way.setAttribute(action);
			 Attribute visible = new Attribute("visible","true");			 
			 way.setAttribute(visible);
			 
			 //ajout des noeuds dans l'criture du chemin
			 List<Node> np = r.getWays().get(i).getNodes();
			 for(int j=0; j<np.size();j++){
				 Element node = new Element("nd");
				 Attribute ref = new Attribute("ref",Integer.toString(np.get(j).getId()));
				 node.setAttribute(ref);
				 way.addContent(node);
			 }
			 
			 //Ajout des tags pour les chemins
			 List<Tag_OSM> l_tag = r.getWays().get(i).getTags();
			 for(int j=0; j<l_tag.size();j++){
				 Element tag = new Element("tag");
				 Attribute k = new Attribute("k",l_tag.get(j).getType().toString());
				 tag.setAttribute(k);
				 Attribute v = new Attribute("v",l_tag.get(j).getValue().toString());
				 tag.setAttribute(v);
				 way.addContent(tag);
			 }
			 
		 	osm.addContent(way);
		 }
	       
	 	//Ajout de la relation avec ses attributs
		 //Boucle sur la liste de relation
	 	Element relation = new Element("relation");
		Attribute id = new Attribute("id",Integer.toString(r.getId()));
		relation.setAttribute(id);
		Attribute action = new Attribute("action","modify");
		relation.setAttribute(action);
		Attribute visible = new Attribute("visible","true");			 
		relation.setAttribute(visible);
		
		//Ajout des chemins (members)
		for(int i=0; i<r.getWays().size(); i++){
			Element member = new Element("member");
			Attribute type = new Attribute("type","way");
			member.setAttribute(type);
			Attribute ref = new Attribute("ref",Integer.toString(r.getWays().get(i).getId()));
			member.setAttribute(ref);
			Attribute role = new Attribute("role","");			 
			member.setAttribute(role);
			relation.addContent(member);
		}
		
		//Ajout des tags d'une relation 
		 for(int j=0; j<r.getTags().size();j++){
			 Element tag = new Element("tag");
			 Attribute k = new Attribute("k",r.getTags().get(j).getType().toString());
			 tag.setAttribute(k);
			 Attribute v = new Attribute("v",r.getTags().get(j).getValue().toString());
			 tag.setAttribute(v);
			 relation.addContent(tag);
		 }
		 
		osm.addContent(relation);
 
		//Les deux mthodes qui suivent seront dfinies plus loin dans l'article
		//affiche(document); //Debug pour vrifier le contenu du document xml
		String depart = ((NodeP) r.getNodes().get(0)).getName();
		String sortie = ((NodeP) r.getNodes().get(r.getNodes().size()-1)).getName();
		Date date =  new Date();
		//Sauvegarde du fichier XML
		enregistre("./mnt/sdcard/Trodometre/"+ depart + "-" + sortie + "-" + date.getTime() + ".xml", document);
		//enregistre(Environment.getExternalStorageState().toString(), document);	           
	}
	
	/**
	 * Put the Document in LogCat
	 * @param document 
	 */
	static void affiche(Document document)
	{
	   try
	   {
	      //On utilise ici un affichage classique avec getPrettyFormat()
	      XMLOutputter sortie = new XMLOutputter(Format.getPrettyFormat());
	      sortie.output(document, System.out);
	   }
	   catch (java.io.IOException e){}
	}
	
	/**
	 * Save a Document in a file  
	 * @param fichier 
	 * @param document
	 */
	static void enregistre(String fichier, Document document)
	{
	   try
	   {
	      XMLOutputter sortie = new XMLOutputter(Format.getPrettyFormat());
	      FileOutputStream f = new FileOutputStream(fichier);
	      OutputStreamWriter osw = new OutputStreamWriter(f);
	      sortie.output(document, osw);
	      osw.close();
	      f.close();
	      Log.d("TAG","Ecriture XML : " + fichier);
	   }
	   catch (java.io.IOException e){}
	}
}
