package sensor;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.util.Log;

import com.engine.Engine;

public class Compass implements SensorEventListener {
	private static final String TAG = "Compass";

	private SensorManager sensorManager;
	private Sensor gsensor;
	private Sensor msensor;
	private float[] mGravity = new float[3];
	private float[] mGeomagnetic = new float[3];
	private float azimuth = 0f;
	private Engine e ;
	
	private float[] num_tab;
	private boolean new_node = false;
	private boolean start = false;
	
	//Variables calcul d'orientation
	private int num =0; //Variable pour la comparaison de num valeur (pour variance et moyenne)
	private float degree ;

	//nouveau fonctionnement à 45° près avec valeur de départ
	private float direction_d; //Direction du batiment en modulo	
	private float direction = 0;
	private float direction_prec = 0;

	public Compass(Context context,Engine engine, float dist) {
		sensorManager = (SensorManager) context.getSystemService(Context.SENSOR_SERVICE);
		gsensor = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
		msensor = sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);
		e = engine;
		num_tab = new float[50];
		//modulo 45 pour une précision de 45°
		direction_d = dist%45;
	}
	
	public void start() {
		direction = 0 ;
		new_node = false ;
		sensorManager.registerListener(this, gsensor,
				SensorManager.SENSOR_DELAY_UI);
		sensorManager.registerListener(this, msensor,
				SensorManager.SENSOR_DELAY_UI);
	}

	public void stop() {
		sensorManager.unregisterListener(this);
	}
	
	/**
	 * 
	 * @param tab
	 * @return average values ​in the table
	 */
	float moyenne_tab(float tab[]){
		float total = 0;
		for(int i=0; i<tab.length;i++){
			total += tab[i];
		}
		return total/tab.length;
	}
	/**
	 * 
	 * @param tab
	 * @param moy
	 * @return value of the variance in the table
	 */
	float cal_variance(float[] tab, float moy){
		float var = 0;
		for(int i=0; i<tab.length; i++){
			var += Math.pow(tab[0]-moy, 2);
		}
		return var/tab.length;
	}

	/**
	 * Cette méthode appelée à temps régulier calcul la direction du téléphone
	 * La direction est calculée à l'aide de deux capteurs, l'accéléromètre et la boussole
	 * La moyenne et la variance sont calculées sur les 50 dernières valeurs
	 * En fonction de la direction, +/- 45, 90... sont ajoutés à la valeur initial du bâtiment 
	 */
	@Override
	public void onSensorChanged(SensorEvent event) {
		final float alpha = 0.97f;

		synchronized (this) {
			if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {

				mGravity[0] = alpha * mGravity[0] + (1 - alpha)
						* event.values[0];
				mGravity[1] = alpha * mGravity[1] + (1 - alpha)
						* event.values[1];
				mGravity[2] = alpha * mGravity[2] + (1 - alpha)
						* event.values[2];

				// mGravity = event.values;
			}

			if (event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD) {
				// mGeomagnetic = event.values;

				mGeomagnetic[0] = alpha * mGeomagnetic[0] + (1 - alpha)
						* event.values[0];
				mGeomagnetic[1] = alpha * mGeomagnetic[1] + (1 - alpha)
						* event.values[1];
				mGeomagnetic[2] = alpha * mGeomagnetic[2] + (1 - alpha)
						* event.values[2];
			}

			float R[] = new float[9];
			float I[] = new float[9];
			boolean success = SensorManager.getRotationMatrix(R, I, mGravity,
					mGeomagnetic);
			if (success) {
				float orientation[] = new float[3];
				SensorManager.getOrientation(R, orientation);
				// Log.d(TAG, "azimuth (rad): " + azimuth);
				azimuth = (float) Math.toDegrees(orientation[0]); // orientation
				azimuth = (azimuth + 360) % 360;
				// Log.d(TAG, "azimuth (deg): " + azimuth);
				//Log.i(TAG, "will set rotation from " + currectAzimuth + " to "+ azimuth);
			}
		}
	
		degree = azimuth;
		//Ajout dans un tableau des 50 dernières valeurs relevées
		num_tab[num] = degree;
		num = (num + 1)%50;
		
		//Calcul de variance sur le tableau des 50 dernières valeurs relevées
		float moyenne = moyenne_tab(num_tab);
		float var = cal_variance(num_tab, moyenne);
			
//		Log.d("direction", ""+direction);
//		Log.d("degree", ""+degree);
		
		if(new_node && var > 0.1){
			
			direction_prec = direction;
			
			//Vérification de la nouvelle direction
			if((moyenne > Math.max(337.5+direction_d,360) && moyenne < 360)
				||(moyenne > 0 && moyenne < 22.5+direction_d)){
				direction = (direction_d) %360;
				new_node = false;
			}if(moyenne > 22.5+direction_d && moyenne < 67.5+direction_d){
				direction = (direction_d + 45) %360;
				new_node = false;
			}else if(moyenne > 67.5+direction_d && moyenne < 112.5+direction_d){
				direction = (direction_d + 90) %360;
				new_node = false;
			}else if(moyenne > 112.5+direction_d && moyenne < 157.5+direction_d){
				direction = (direction_d + 135) %360;
				new_node = false;
			}else if(moyenne > 157.5+direction_d && moyenne < 202.5+direction_d){
				direction = (direction_d + 180) %360;
				new_node = false;
			}else if(moyenne > 202.5+direction_d && moyenne < 247.5+direction_d){
				direction = (direction_d + 225) %360;
				new_node = false;
			}else if(moyenne > 247.5+direction_d && moyenne < 292.5+direction_d){
				direction = (direction_d + 270) %360;
				new_node = false;
			}else if(moyenne > 292.5+direction_d && moyenne < Math.max(337.5+direction_d,360)){
				direction = (direction_d + 315) %360;
				new_node = false;
			}
			//création du node, puis sauvegarde de l'ancienne valeur
			if(direction != direction_prec && new_node == false  && start){
				e.new_node(direction_prec);
				e.updateDisplayDirection("Compass : " + Float.toString(direction));				
			}
		}
		
		//Code pour calcul d'orientation direct avec degree mais manque de précision à certain moment
		/*if(direction < 70 || direction > 290 ){
			if((var < 0.05 && moyenne< (direction+290)%360 && moyenne > (direction+70)%360) && new_node==true){
				e.new_node(direction);
				new_node = false;
				Log.d("changement1", ""+direction);
				e.updateDisplayDirection("Compass : " + Float.toString(direction));
			}
		}
		else{
			if( (moyenne > (direction+70)%360 || (moyenne < (direction-70)%360)) && new_node==true){
				e.new_node(direction);
				new_node = false;
				Log.d("changement2", ""+direction);
				e.updateDisplayDirection("Compass : " + Float.toString(direction));
			}
		}*/
			
		/*
		 * On attend une variance proche de 0, 
		 * On attend que le noeud précédent soit créé,
		 * On attend que la boussole se calibre (degree == 0)
		 */

		if(var<0.1 && new_node == false && degree != 0){	
			//Log.d("variance", ""+var);
			//Log.d("moyenne", ""+moyenne);
			//Log.d("direction", ""+direction);
			//Valeur de direction correct
			new_node = true;
		}
		e.updateDisplayDirection("Compass : " + Float.toString(direction));
		e.updateDisplayDistance();
	}

	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
	}
	
	/**
	 * Enable to know if mesure is launch
	 * @param b
	 */
	public void set_start(boolean b){
		start = b;
	}
	
	/**
	 * 
	 * @return the current direction 
	 */
	public float getBearing(){
		//return azimuth;
		return direction;
	}
}
